<?php
/**
 * Implements _webform_defaults_component().
 */
function _webform_defaults_address() {
  return array(
    'name' => '',
    'form_key' => NULL,
    'required' => FALSE,
    'pid' => 0,
    'weight' => 0,
    'extra' => array(
      'po_box' => FALSE,
      'addresscheck' => 'none',
      'city_options' => 'textfield',
      'cities_piped_list' => '',
      'placeholders' => FALSE,
      'private' => FALSE,
    ),
  );
}

/**
 * Implements _webform_edit_component().
 */
function _webform_edit_address($component) {
  $form = array();

  $form['extra']['po_box'] = array(
    '#type' => 'checkbox',
    '#title' => t('PO Box'),
    '#description' => t('Check this if this is a PO box address. Street name, number and addition will dissapear and PO box field becomes available. NOTE: address checks are not available with a PO box.'),
    '#default_value' => $component['extra']['po_box'],
  );
  $addresscheck_options = array();
  $addresscheck_options['none'] = t('None - no checks are performed');
  $dvg_postcodenl_key = variable_get('dvg_postcodenl_key');
  if (!empty($dvg_postcodenl_key)) {
    $addresscheck_options['postcodenl'] = t('Postcode.nl - only addresses in the Netherlands are allowed');
  }
  $dvg_stuf_bg_aoa_api_url = variable_get('dvg_stuf_bg_aoa_api_url');
  if (function_exists('dvg_stuf_bg_adresgegevens') && !empty($dvg_stuf_bg_aoa_api_url)) {
    $addresscheck_options['stuf-bg'] = t('StUF-BG Addresscheck - only addresses in council are allowed');
  }
  $form['extra']['addresscheck'] = array(
    '#type' => 'select',
    '#title' => t('Address check'),
    '#description' => t('Optional. Select the service to complete streetname and city based on postal code + house number.'),
    '#options' => $addresscheck_options,
    '#default_value' => $component['extra']['addresscheck'],
  );
  $form['extra']['city_options'] = array(
    '#type' => 'select',
    '#title' => t('City options'),
    '#description' => t('Do you want to display the city field as a select list or free textfield? NOTE: address checks are not available if predefined list is used.'),
    '#options' => array(
      'textfield' => t('Textfield'),
      'select' => t('Predefined list'),
    ),
    '#default_value' => $component['extra']['city_options'],
  );
  $form['extra']['cities_piped_list'] = array(
    '#type' => 'textarea',
    '#title' => t('Predefined list of cities'),
    '#description' => t('If you choose predefined list above, please provide a piped list (key|value) of all available cities.'),
    '#default_value' => $component['extra']['cities_piped_list'],
  );
  $form['extra']['placeholders'] = array(
    '#type' => 'checkbox',
    '#title' => t('Placeholders'),
    '#description' => t('Use a placeholder instead of the label. The placeholder will be shown in the field until the user starts entering a value.'),
    '#default_value' => $component['extra']['placeholders'],
  );
  $form['extra']['default_values'] = array(
    '#type' => 'fieldset',
    '#title' => t('Default values'),
  );
  $form['extra']['default_values']['postal_code'] = array(
    '#type' => 'textfield',
    '#title' => t('Default value') . ' : ' . t('Postal code'),
    '#default_value' => isset($component['extra']['default_values']['postal_code']) ? $component['extra']['default_values']['postal_code'] : '',
    '#description' => t('The default value of the field.') . theme('webform_token_help'),
  );
  $form['extra']['default_values']['house_number'] = array(
    '#type' => 'textfield',
    '#title' => t('Default value') . ' : ' . t('House number'),
    '#default_value' => isset($component['extra']['default_values']['house_number']) ? $component['extra']['default_values']['house_number'] : '',
    '#description' => t('The default value of the field.') . theme('webform_token_help'),
  );
  $form['extra']['default_values']['house_letter'] = array(
    '#type' => 'textfield',
    '#title' => t('Default value') . ' : ' . t('House letter'),
    '#default_value' => isset($component['extra']['default_values']['house_letter']) ? $component['extra']['default_values']['house_letter'] : '',
    '#description' => t('The default value of the field.') . theme('webform_token_help'),
  );
  $form['extra']['default_values']['house_number_addition'] = array(
    '#type' => 'textfield',
    '#title' => t('Default value') . ' : ' . t('Number addition'),
    '#default_value' => isset($component['extra']['default_values']['house_number_addition']) ? $component['extra']['default_values']['house_number_addition'] : '',
    '#description' => t('The default value of the field.') . theme('webform_token_help'),
  );
  $form['extra']['default_values']['street_name'] = array(
    '#type' => 'textfield',
    '#title' => t('Default value') . ' : ' . t('Street name'),
    '#default_value' => isset($component['extra']['default_values']['street_name']) ? $component['extra']['default_values']['street_name'] : '',
    '#description' => t('The default value of the field.') . theme('webform_token_help'),
  );
  $form['extra']['default_values']['po_box'] = array(
    '#type' => 'textfield',
    '#title' => t('Default value') . ' : ' . t('PO Box'),
    '#default_value' => isset($component['extra']['default_values']['po_box']) ? $component['extra']['default_values']['po_box'] : '',
    '#description' => t('The default value of the field.') . theme('webform_token_help'),
  );
  $form['extra']['default_values']['city'] = array(
    '#type' => 'textfield',
    '#title' => t('Default value') . ' : ' . t('City'),
    '#default_value' => isset($component['extra']['default_values']['city']) ? $component['extra']['default_values']['city'] : '',
    '#description' => t('The default value of the field.') . theme('webform_token_help'),
  );
  $form['extra']['default_values']['council_code'] = array(
    '#type' => 'textfield',
    '#title' => t('Default value') . ' : ' . t('Council code'),
    '#default_value' => isset($component['extra']['default_values']['council_code']) ? $component['extra']['default_values']['council_code'] : '',
    '#description' => t('The default value of the field.') . theme('webform_token_help'),
  );
  return $form;
}

/**
 * Implements _webform_render_component().
 */
function _webform_render_address($component, $value = NULL, $filter = TRUE) {
  // Wrap the form in a container, in order to correctly apply the weight.
  $elements = array(
    '#type' => 'container',
    '#attached' => array(
      'css' => array(
        drupal_get_path('module', 'dvg_webform_components') . '/css/address.css',
      ),
      'js' => array(
        drupal_get_path('module', 'dvg_webform_components') . '/js/address.js',
      ),
    ),
    '#weight' => $component['weight'],
    '#theme_wrappers' => array('webform_element', 'ife_form_element'),
    '#webform_component' => $component,
  );
  $elements['postal_code'] = array(
    '#type' => 'textfield',
    '#title' => t('Postal code'),
    '#title_display' => $component['extra']['title_display'] ? $component['extra']['title_display'] : 'before',
    '#default_value' => isset($value['postal_code']) ? $value['postal_code'] : (isset($component['extra']['default_values']['postal_code']) ? token_replace($component['extra']['default_values']['postal_code']) : ''),
    '#required' => $component['required'],
    '#attributes' => array(),
    '#size' => 10,
    '#theme_wrappers' => array('webform_element', 'ife_form_element'),
    '#webform_component' => array('type' => $component['type'] . '_postal_code') + $component,
  );
  if (empty($component['extra']['po_box'])) {
    $elements['house_number'] = array(
      '#type' => 'textfield',
      '#title' => t('House number'),
      '#title_display' => $component['extra']['title_display'] ? $component['extra']['title_display'] : 'before',
      '#default_value' => isset($value['house_number']) ? $value['house_number'] : (isset($component['extra']['default_values']['house_number']) ? token_replace($component['extra']['default_values']['house_number']) : ''),
      '#required' => $component['required'],
      '#attributes' => array(),
      '#size' => 10,
      '#theme_wrappers' => array('webform_element'),
      '#webform_component' => array('type' => $component['type'] . '_house_number') + $component,
    );
    $elements['house_letter'] = array(
      '#type' => 'textfield',
      '#title' => t('House letter'),
      '#title_display' => $component['extra']['title_display'] ? $component['extra']['title_display'] : 'before',
      '#default_value' => isset($value['house_letter']) ? $value['house_letter'] : (isset($component['extra']['default_values']['house_letter']) ? token_replace($component['extra']['default_values']['house_letter']) : ''),
      '#required' => FALSE,
      '#attributes' => array(),
      '#size' => 6,
      '#theme_wrappers' => array('webform_element'),
      '#webform_component' => array('type' => $component['type'] . '_house_letter') + $component,
    );
    $elements['house_number_addition'] = array(
      '#type' => 'textfield',
      '#title' => t('Number addition'),
      '#title_display' => $component['extra']['title_display'] ? $component['extra']['title_display'] : 'before',
      '#default_value' => isset($value['house_number_addition']) ? $value['house_number_addition'] : (isset($component['extra']['default_values']['house_number_addition']) ? token_replace($component['extra']['default_values']['house_number_addition']) : ''),
      '#required' => FALSE,
      '#attributes' => array(),
      '#size' => 10,
      '#theme_wrappers' => array('webform_element'),
      '#webform_component' => array('type' => $component['type'] . '_house_number_addition') + $component,
    );
    $elements['block'] = array(
      '#type' => 'container',
      '#theme_wrappers' => array('webform_element', 'ife_form_element'),
      '#webform_component' => array('type' => $component['type'] . '_block') + $component,
    );
    $elements['street_name'] = array(
      '#type' => 'textfield',
      '#title' => t('Street name'),
      '#title_display' => $component['extra']['title_display'] ? $component['extra']['title_display'] : 'before',
      '#default_value' => isset($value['street_name']) ? $value['street_name'] : (isset($component['extra']['default_values']['street_name']) ? token_replace($component['extra']['default_values']['street_name']) : ''),
      '#required' => $component['required'],
      '#attributes' => array(),
      '#theme_wrappers' => array('webform_element'),
      '#webform_component' => array('type' => $component['type'] . '_street_name') + $component,
    );
    $elements['council_code'] = array(
      '#type' => 'hidden',
      '#title' => t('Council code'),
      '#title_display' => 'before',
      '#default_value' => isset($value['council_code']) ? $value['council_code'] : (isset($component['extra']['default_values']['council_code']) ? token_replace($component['extra']['default_values']['council_code']) : ''),
      '#required' => FALSE,
      '#theme' => 'webform_display_council_code',
      '#attributes' => array(),
    );
    if ($component['extra']['addresscheck'] == 'stuf-bg') {
      $elements['street_name']['#attributes']['readonly'] = 'readonly';
    }
  } else {
    $elements['po_box'] = array(
      '#type' => 'textfield',
      '#title' => t('PO Box'),
      '#title_display' => $component['extra']['title_display'] ? $component['extra']['title_display'] : 'before',
      '#default_value' => isset($value['po_box']) ? $value['po_box'] : (isset($component['extra']['default_values']['po_box']) ? token_replace($component['extra']['default_values']['po_box']) : ''),
      '#required' => $component['required'],
      '#attributes' => array(),
      '#size' => 12,
      '#theme_wrappers' => array('webform_element'),
      '#webform_component' => array('type' => $component['type'] . '_po_box') + $component,
    );
  }
  if ($component['extra']['city_options'] == 'textfield') {
    $elements['city'] = array(
      '#type' => 'textfield',
      '#title' => t('City'),
      '#title_display' => $component['extra']['title_display'] ? $component['extra']['title_display'] : 'before',
      '#default_value' => isset($value['city']) ? $value['city'] : (isset($component['extra']['default_values']['city']) ? token_replace($component['extra']['default_values']['city']) : ''),
      '#required' => $component['required'],
      '#attributes' => array(),
      '#theme_wrappers' => array('webform_element'),
      '#webform_component' => array('type' => $component['type'] . '_city') + $component,
    );
    if ($component['extra']['addresscheck'] == 'stuf-bg') {
      $elements['city']['#attributes']['readonly'] = 'readonly';
    }
  }

  if ($component['extra']['city_options'] == 'select') {
    $options = list_extract_allowed_values($component['extra']['cities_piped_list'], 'list_text', FALSE);
    $elements['city'] = array(
      '#type' => 'select',
      '#title' => t('City'),
      '#options' => $options,
      '#title_display' => $component['extra']['title_display'] ? $component['extra']['title_display'] : 'before',
      '#default_value' => isset($value['city']) ? $value['city'] : (isset($component['extra']['default_values']['city']) ? token_replace($component['extra']['default_values']['city']) : ''),
      '#required' => $component['required'],
      '#attributes' => array(),
      '#theme_wrappers' => array('webform_element'),
      '#webform_component' => array('type' => $component['type'] . '_city') + $component,
    );
  }
  $dvg_stuf_bg_aoa_api_url = variable_get('dvg_stuf_bg_aoa_api_url');
  if ($component['extra']['addresscheck'] == 'stuf-bg' && empty($dvg_stuf_bg_aoa_api_url)) {
    watchdog('dvg_webform_components', 'Addresscheck set to stuf-bg but variable dvg_stuf_bg_aoa_api_url not set ', array(), WATCHDOG_WARNING, $link = NULL);
  }
  $dvg_postcodenl_key = variable_get('dvg_postcodenl_key');
  if ($component['extra']['addresscheck'] == 'postcodenl' && empty($dvg_postcodenl_key)) {
    watchdog('dvg_webform_components', 'Addresscheck set to postcode.nl but variable dvg_postcodenl_key not set ', array(), WATCHDOG_WARNING, $link = NULL);
  }
  $dvg_postcodenl_secret = variable_get('dvg_postcodenl_secret');
  if ($component['extra']['addresscheck'] == 'postcodenl' && empty($dvg_postcodenl_secret)) {
    watchdog('dvg_webform_components', 'Addresscheck set to postcode.nl but variable dvg_postcodenl_secret not set ', array(), WATCHDOG_WARNING, $link = NULL);
  }

  $ajax = FALSE;
  if ($component['extra']['addresscheck'] != 'none' && $component['extra']['city_options'] != 'select' && !$component['extra']['po_box']) {
    $ajax = TRUE;
  }

  // ajax calls for addresschecks only added if :
  // - selected service
  // - cities not a list
  // - not a po box
  if ($ajax) {
    $elements['postal_code']['#ajax'] = array(
      'event' => 'change',
      'callback' => '_address_ajax_callback',
    );
    $elements['house_number']['#ajax'] = array(
      'event' => 'change',
      'callback' => '_address_ajax_callback',
    );
    if ($component['extra']['addresscheck'] == 'stuf-bg') {
      $elements['house_letter']['#ajax'] = array(
        'event' => 'change',
        'callback' => '_address_ajax_callback',
      );
      $elements['house_number_addition']['#ajax'] = array(
        'event' => 'change',
        'callback' => '_address_ajax_callback',
      );

    }
    $elements['street_name']['#attributes']['class'] = array(drupal_html_class('ajax-callback-street-name--' . $component['form_key']));
    $elements['city']['#attributes']['class'] = array(drupal_html_class('ajax-callback-city--' . $component['form_key']));
  }

  $elements['#element_validate'] = array('dvg_webform_components_address_validate_addresscheck');

  // Set the label as a place holder.
  if ($component['extra']['placeholders']) {

    $elements['po_box']['#title_display'] = 'invisible';
    $elements['po_box']['#attributes']['placeholder'] = t('PO Box');

    $elements['postal_code']['#title_display'] = 'invisible';
    $elements['postal_code']['#attributes']['placeholder'] = t('Postal code');

    $elements['house_number']['#title_display'] = 'invisible';
    $elements['house_number']['#attributes']['placeholder'] = t('House number');

    $elements['house_letter']['#title_display'] = 'invisible';
    $elements['house_letter']['#attributes']['placeholder'] = t('House letter');

    $elements['house_number_addition']['#title_display'] = 'invisible';
    $elements['house_number_addition']['#attributes']['placeholder'] = t('Addition');

    $elements['street_name']['#title_display'] = 'invisible';
    $elements['street_name']['#attributes']['placeholder'] = t('Street name');

    $elements['city']['#title_display'] = 'invisible';
    $elements['city']['#attributes']['placeholder'] = t('City');
  }

  return $elements;
}

/**
 * Retrieve street name and city belonging to postal code and house number.
 *
 * Callback for _webform_render_address_auto().
 *
 * NB: Normally, this function should be kept as simple as possible (i.e., just
 *     return the portion of the form that should be adjusted). However, since
 *     Webform doesn't provide the $form_state in a simple manner and we only
 *     provide additional functionallity to help the visitor, we allow us to do
 *     a little bit more in this function.. (otherwise we have to use the
 *     hook_form_alter() function to add AJAX components).
 */
function _address_ajax_callback($form, $form_state) {
  // To authenticate to the postcode.nl API, we need the login information.

  $service = $form_state['triggering_element']['#webform_component']['extra']['addresscheck'];

  // Get the key used for this Webform component.
  $form_key = $form_state['triggering_element']['#webform_component']['form_key'];

  // Get the provided postal code and house number.
  $values = $form_state['values'];
  foreach ($form_state['triggering_element']['#parents'] as $parent) {
    $values = $values[$parent];

    if ($parent == $form_key) {
      break;
    }
  }

  $errors = array();
  $postal_code = $values['postal_code'];
  _address_validate_postal_code($postal_code, $form_state['triggering_element']['#webform_component']['required'], $errors);
  if (empty($form_state['triggering_element']['#webform_component']['extra']['po_box'])) {
    $house_number = $values['house_number'];
    $house_letter = $values['house_letter'];
    $house_number_addition = $values['house_number_addition'];
    _address_validate_house_number($house_number, $form_state['triggering_element']['#webform_component']['required'], $errors);
    _address_validate_house_letter($house_letter, $errors);
    _address_validate_house_number_addition($house_number_addition, $errors);
  }
  $commands = array();
  if (empty($errors)) {
    // Perform the lookup using the service.
    $data = _address_autofill($postal_code, $house_number, $house_letter, $house_number_addition, $service);
    // React on the provided information.
    $arguments = array(
      'formKey' => drupal_html_class($form_key),
      'street' => check_plain($data['street']),
      'city' => check_plain($data['city']),
    );
    $arguments = drupal_json_encode($arguments);
    $commands[] = ajax_command_invoke(NULL, 'webformAddressInject', array($arguments));
  }
  if ($service == 'stuf-bg') {
    // Remove the old messages div, clearing existing messages.
    $commands[] = ajax_command_remove('.webform-component--' . $form_key_class . ' .messages.error');
    $location = '.webform-component--' . drupal_html_class($form_key) . ' .webform-component-address-block';
    if (empty($errors)) {
      if (empty($data['street']) && empty($data['city'])) {
        $error = t('This address is not available in %site.', array('%site' => variable_get('site_name', 'this site')));
        $commands[] = ajax_command_after($location, '<div class="messages error messages-inline">' . $error . '</div>');
      }
    }
    else {
      if (count($errors) > 1) {
        $items = array(
          'items' => $errors,
          'title' => '',
          'type' => 'ul',
          'attributes' => array()
        );
        $error = theme_item_list($items);
      }
      else {
        $error = $errors[0];
      }
      $commands[] = ajax_command_after($location, '<div class="messages error messages-inline">' . $error . '</div>');

    }
  }

  return array(
    '#type' => 'ajax',
    '#commands' => $commands,
  );

}

/**
 * Wrapper function which calls different services based on settings.
 *
 * @param string $service
 *   Type of service: currently: postcodenl, aoa (dvg-stuf-bg)
 * @param string $postal_code
 *   The postal code of the address.
 * @param string $house_number
 *   The house number of the address.
 */
function _address_autofill($postal_code, $house_number, $house_letter = '', $house_number_addition = '', $service) {

  $data = array();
  $data['street'] = '';
  $data['city'] = '';

  switch($service) {
    case 'stuf-bg':
      $prefill = dvg_stuf_bg_adresgegevens($postal_code, $house_number, $house_letter, $house_number_addition);
      if ($prefill) {
        $data['street'] = $prefill->{'gor.straatnaam'}->_;
        $data['city'] = $prefill->{'wpl.woonplaatsNaam'}->_;
      }
      break;
    case 'postcodenl':
      $key = variable_get('dvg_postcodenl_key');
      $secret = variable_get('dvg_postcodenl_secret');
      $data = _address_postcode_nl_lookup($key, $secret, $postal_code, $house_number);
      break;
  }
  return $data;
}

/**
 * Retrieve street name and city belonging to postal code and house number.
 *
 * This function makes a call to the postcode.nl API to get the street name and
 * city that is associated with the postal code and house number.
 *
 * @param string $key
 *   The key that should be used to identify at the postcode.nl API service.
 * @param string $secret
 *   The secret that belongs to the $key.
 * @param string $postal_code
 *   The postal code of the address.
 * @param string $house_number
 *   The house number of the address.
 *
 * @return string[]
 *   The name of the street can be found at the index 'street' and the name of
 *   the name of the city at the index 'city' in the array.
 */
function _address_postcode_nl_lookup($key, $secret, $postal_code, $house_number) {
  // First, parse the postal code and house number (which should split the
  // addition).
  $postal_code = _address_parse_postal_code($postal_code);
  list($house_number, $house_number_addition) = _address_parse_house_number($house_number);

  // Both the postal code and the house number should have a value.
  if (empty($postal_code) || empty($house_number)) {
    return array();
  }

  // We prevent making unnecessary lookups by caching results, also wrong
  // lookups!
  $cache_key = 'dvg_address:' . $postal_code . ':' . $house_number . ':' . $house_number_addition;

  // Use the local memory as the first level cache.
  $data = &drupal_static(__FUNCTION__);
  if (!isset($data)) {
    // When not stored in local memory, check the cache stored in the database.
    if ($cache = cache_get($cache_key)) {
      $data = $cache->data;
    }
    else {
      // Construct the URL to perform the lookup with.
      $url = 'https://' . $key . ':' . $secret . '@api.postcode.nl/rest/addresses/' . $postal_code . '/' . $house_number . '/' . $house_number_addition;

      $response = drupal_http_request(url($url));

      // When the lookup went correctly, parse the JSON data.
      if ($response->code == 200) {
        $data = drupal_json_decode($response->data);
      }

      // Store the result in the cache of the database.
      //
      // NB: if the lookup didn't responded in a correct result, we still store
      // a 'mark' in the database. In this way also invalid lookups will be
      // served from cache the next time.
      cache_set($cache_key, $data, 'cache', time() + 900);
    }
  }

  return $data;
}

/**
 * Parse a postal code to be used in the request to postcode.nl API.
 *
 * @param string $postal_code
 *   The postal code that should be normalized.
 *
 * @return string
 *   The normalized postal code.
 */
function _address_parse_postal_code($postal_code) {
  // First, remove any whitespaces.
  $postal_code = str_replace(' ', '', trim($postal_code));

  // If the postal code is in the form of 1234AB, than return the trimmed
  // version.
  if ((boolean) preg_match('~^\d{4}[[:alpha:]]{2}$~', $postal_code)) {
    return $postal_code;
  }

  return '';
}

/**
 * Parse the house number by separating the number from the additional value.
 *
 * @param string $house_number_full
 *   The house number of the address (including an possible addition).
 *
 * @return string[]
 *   Normalized version of the house number, where the first index of the array
 *   contains the number and the second value is the addition.
 */
function _address_parse_house_number($house_number_full) {
  // Remove white spaces before and after the provided house number (including
  // addition).
  $house_number_full = trim($house_number_full);

  $house_number = '';
  $house_number_addition = '';
  if (preg_match('~^(?P<house_number>\d+)[^[:alnum:]]*(?P<addition>[[:alnum:]]*)$~', $house_number_full, $match)) {
    $house_number = $match['house_number'];
    $house_number_addition = !empty($match['addition']) ? $match['addition'] : '';
  }

  return array($house_number, $house_number_addition);
}

/**
 * Implements _webform_display_component().
 */
function _webform_display_address($component, $value, $format = 'html') {
  return array(
    '#title' => $component['name'],
    '#weight' => $component['weight'],
    '#theme' => 'webform_display_address',
    '#theme_wrappers' => $format == 'html' ? array('webform_element') : array('webform_element_text'),
    '#post_render' => array('webform_element_wrapper'),
    '#component' => $component,
    '#format' => $format,
    '#value' => isset($value) ? address_display_format($value) : '',
  );
}

/**
 * Implements _webform_theme_component().
 */
function _webform_theme_address() {
  return array(
    'webform_display_address' => array(
      'render element' => 'element',
    ),
  );
}

/**
 * Returns HTML for displaying the address.
 *
 * @param array $variables
 *   An associative array containing:
 *   - element: An associative array containing the properties of the element.
 *     Properties used: #format, #value.
 *
 * @see theme_webform_element
 *
 * @ingroup themeable
 */
function theme_webform_display_address(array $variables) {
  $element = $variables['element'];
  $value = $element['#format'] == 'html' ? check_plain($element['#value']) : $element['#value'];

  return $value !== '' ? $value : ' ';
}

/**
 * Implements _webform_table_component().
 */
function _webform_table_address($component, $value) {
  $street = $value['street_name'] . ' ' . $value['house_number'];
  if (isset($value['house_letter'])) {
    $street .= $value['house_letter'];
  }
  if (isset($value['house_number_addition'])) {
    $street .=  ' ' . $value['house_number_addition'];
  }
  return check_plain($street) . '<br/>' . check_plain($value['postal_code'] . ' ' . $value['city']);

}

/**
 * Implements _webform_csv_headers_component().
 */
function _webform_csv_headers_address($component, $export_options) {
  $header = array();
  $header[0] = array('');
  $header[1] = array($component['name']);
  $header[2] = array();

  $header[2][] = t('Street name');
  $header[2][] = t('House number');
  $header[2][] = t('Postal code');
  $header[2][] = t('City');

  return $header;
}

/**
 * Implements _webform_csv_data_component().
 */
function _webform_csv_data_address($component, $export_options, $value) {
  $return = array();

  $return[] = $value['street_name'];
  $return[] = $value['house_number'];
  $return[] = $value['postal_code'];
  $return[] = $value['city'];

  return $return;
}

/**
 * Format the address in a textual representation.
 *
 * @param string[] $value
 *   An associative array containing the address information (i.e., street name,
 *   house number, postal code and the city).
 *
 * @return string
 *   A formatted string representing the address.
 */
function address_display_format(array $value) {
  $street = $value['street_name'] . ' ' . $value['house_number'];
  if (isset($value['house_letter'])) {
    $street .= $value['house_letter'];
  }
  if (isset($value['house_number_addition'])) {
    $street .=  ' ' . $value['house_number_addition'];
  }
  return check_plain($street . ', ' . $value['postal_code'] . ' ' . $value['city']);
}

/**
 * Validate entire form, need
 */
function dvg_webform_components_address_validate_addresscheck($element, &$form_state) {
  $errors = array();
  $postal_code = $element['postal_code']['#value'];
  _address_validate_postal_code($postal_code, $element['postal_code']['#webform_component']['required'], $errors);
  if (!empty($component['#webform_component']['extra']['po_box'])) {
    $po_box = $element['po_box']['#value'];
  } else {
    $house_number = $element['house_number']['#value'];
    $house_letter = $element['house_letter']['#value'];
    $house_number_addition = $element['house_number_addition']['#value'];
    _address_validate_house_number($house_number, $element['house_number']['#webform_component']['required'], $errors);
    _address_validate_house_letter($house_letter, $errors);
    _address_validate_house_number_addition($house_number_addition, $errors);
  }

  if (isset($po_box)) {
    _address_validate_po_box($po_box, $element['po_box']['#webform_component']['required'], $errors);
  }

  $city = $element['city']['#value'];
  $street_name = $element['street_name']['#value'];

  if ($element['#webform_component']['extra']['addresscheck'] != 'none' && $element['#webform_component']['extra']['city_options'] != 'select' && !$element['#webform_component']['extra']['po_box']) {
    $service = $element['#webform_component']['extra']['addresscheck'];
    // perform the lookup once again
    $data = _address_autofill($postal_code, $house_number, $house_letter, $house_number_addition, $service);
    // React on the provided information.
    if (empty($errors) && (empty($data) || (empty($data['street']) || empty($data['city'])) || $data['street'] != $street_name || $data['city'] != $city)) {
      switch ($service) {
        case 'postcodenl';
          $errors[] = t('This address is not available in the Netherlands.');
        break;
        case 'stuf-bg';
          $errors[] = t('This address is not available in %site.', array('%site' => variable_get('site_name', 'this site')));
        break;
      }
    }
  }

  if (!empty($errors)) {
    $form_keys = $element['block']['#parents'];
    array_shift($form_keys);
    if (count($errors) > 1) {
      $items = array(
        'items' => $errors,
        'title' => '',
        'type' => 'ul',
        'attributes' => array()
      );
      $errors = theme_item_list($items);
    }
    else {
      $errors = $errors[0];
    }
    form_set_error('submitted][' . implode('][', $form_keys), $errors);
  }
}

/**
 * Postal code validation
 */
function _address_validate_postal_code($postal_code, $required, &$errors) {
  if (!empty($postal_code)) {
    preg_match('/^([1-9][0-9]{3}) ?([A-Za-z]{2})$/', $postal_code, $matches);
    if ( empty($matches) || (isset($matches[2]) && in_array($matches[2], array('SS','SD','SA'), TRUE)) ) {
      $errors[] = t('Invalid postal code. Postal codes in the Netherlands are like "9999 AA". They never start with zero and the letters are never "SS", "SD", or "SA".');
    }
  } elseif ($required) {
    $errors[] = t('Please enter a postal code.');
  }
}

/**
 * Validation handler for house number.
 */
function _address_validate_house_number($house_number, $required, &$errors) {
  $value = trim($house_number);
  if (!empty($value) && !preg_match('/^[1-9]{1}[0-9]{0,4}$/i', $value)) {
    $errors[] = t('The house number has to be an integer in the range 1 till 99999');
  } elseif (empty($value) && $required) {
    $errors[] = t('Please enter a house number.');
  }
}

/**
 * Validation handler for house letter.
 */
function _address_validate_house_letter($house_letter, &$errors) {
  $value = trim($house_letter);
  if (!empty($value) && !preg_match('/^[a-z]{0,1}$/i', $value)) {
    $errors[] = t('The house letter is max 1 character and exists of the letters (A till Z)');
  }
}

/**
 * Validation handler for house number remark / addition.
 */
function _address_validate_house_number_addition($house_number_addition, &$errors) {
  $value = trim($house_number_addition);
  if (!empty($value) && !preg_match('/^[a-z1-9]{0,4}$/i', $value)) {
    $errors[] = t('The house number remark is max 4 characters and exists of the letters (A till Z) and/or digits (0 till 9)');
  }
}

/**
 * Validation handler for po box number.
 */
function _address_validate_po_box($po_box, $required) {
  $value = trim($po_box);
  if (!empty($value) && !preg_match('/^[1-9]{1}[0-9]{0,4}$/i', $value)) {
    $errors[] = t('The PO box has to be an integer in the range 1 till 99999');
  } elseif (empty($value) && $required) {
    $errors[] = t('Please enter a PO box number.');
  }
}

/**
 * Custom Theme function for council code.
 */
function theme_webform_display_council_code($variables) {
  return '';
}